@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2012-2013  Daniel Pavel
 ##
@@ -17,8 +16,6 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from logging import DEBUG as _DEBUG
 from logging import getLogger
 
@@ -79,7 +76,6 @@ def _look_for_application_icons():
 
 
 _default_theme = None
-_use_symbolic_icons = False
 
 
 def _init_icon_paths():
@@ -90,18 +86,20 @@ def _init_icon_paths():
     _default_theme = Gtk.IconTheme.get_default()
     for p in _look_for_application_icons():
         _default_theme.prepend_search_path(p)
+        break  # only prepend one path - that's sufficient
     if _log.isEnabledFor(_DEBUG):
         _log.debug('icon theme paths: %s', _default_theme.get_search_path())
 
-    if gtk.prefer_symbolic_battery_icons:
-        if _default_theme.has_icon('battery-good-symbolic'):
-            global _use_symbolic_icons
-            _use_symbolic_icons = True
-            return
-        else:
+    if gtk.battery_icons_style == 'symbolic':
+        global TRAY_OKAY
+        TRAY_OKAY = TRAY_INIT  # use monochrome tray icon
+        if not _default_theme.has_icon('battery-good-symbolic'):
             _log.warning('failed to detect symbolic icons')
-    if not _default_theme.has_icon('battery-good'):
-        _log.warning('failed to detect icons')
+            gtk.battery_icons_style = 'regular'
+    if gtk.battery_icons_style == 'regular':
+        if not _default_theme.has_icon('battery-good'):
+            _log.warning('failed to detect icons')
+            gtk.battery_icons_style = 'solaar'
 
 
 #
@@ -129,10 +127,12 @@ def _battery_icon_name(level, charging):
     _init_icon_paths()
 
     if level is None or level < 0:
-        return 'battery-missing' + ('-symbolic' if _use_symbolic_icons else '')
+        return 'battery-missing' + ('-symbolic' if gtk.battery_icons_style == 'symbolic' else '')
 
-    level_name = _first_res(level, ((90, 'full'), (50, 'good'), (20, 'low'), (5, 'caution'), (0, 'empty')))
-    return 'battery-%s%s%s' % (level_name, '-charging' if charging else '', '-symbolic' if _use_symbolic_icons else '')
+    level_name = _first_res(level, ((90, 'full'), (30, 'good'), (20, 'low'), (5, 'caution'), (0, 'empty')))
+    return 'battery-%s%s%s' % (
+        level_name, '-charging' if charging else '', '-symbolic' if gtk.battery_icons_style == 'symbolic' else ''
+    )
 
 
 #
@@ -169,6 +169,8 @@ def device_icon_set(name='_', kind=None):
                 names += ('input-mouse', 'input-tablet')
             elif str(kind) == 'trackball':
                 names += ('input-mouse', )
+            elif str(kind) == 'headset':
+                names += ('audio-headphones', 'audio-headset')
             names += ('input-' + str(kind), )
         # names += (name.replace(' ', '-'),)
 
